;$MOD252
DA0             BIT P1.0
DA1             BIT P1.1
DA2             BIT P1.2
CS0             BIT P1.3
CS1             BIT P1.4
DIOR            BIT P1.5
DIOW            BIT P1.6
IORDY           BIT P3.1
INTRQ           BIT P3.2
RESET           BIT P3.3
;PLAY           BIT P3.4
;STOP           BIT P3.5
;REV            BIT P3.6
;FWD            BIT P3.7
;PAUSE          BIT P1.7
DDL             EQU P0
DDH             EQU P2
DatabusL        EQU 050H
DatabusH        EQU 051H
Byte_Count      EQU 054H
Audio_Status    EQU 055H	;Sub-Q Channel Current Audio Status
Track           EQU 056H	;Current Track Number
Total_Tracks    EQU 058H	;Track to scan to
Button_State    EQU 059H	;Last state of the buttons => Port 3
Button_State1   EQU 060H	;Last state of the buttons => Port 1
M_Field         EQU 061H    	;Minute Field
S_Field         EQU 062H    	;Second Field
F_Field         EQU 063H    	;Frame Field
TOC_Length      EQU 064H    	;TOC data length in bytes
First_Track     EQU 065H   	;First track number on CD
Last_Track      EQU 066H    	;Last track number on CD
;========================== MAIN PROGRAM
		ACALL    Initialise
		ACALL    Start_Unit
		MOV	 A,#000010000B   

loop:   	ACALL    Get_Buttons		
		JMP     loop
;========================== Reset Drive and Initialise Ports
Initialise:
	MOV     P1,#000H     	
	MOV     P3,#000001111B  
	MOV	Button_State,#000H
	MOV	Button_State1,#000H
	SETB    DIOR
	SETB    DIOW
Hard_Reset:
	CLR     RESET           ;Assert Reset-
	ACALL    DELAY
	ACALL    DELAY
	SETB    RESET           ;Negate Reset-  
	ACALL    DELAY
	ACALL    DELAY
Soft_Reset:
	ACALL    Set_Status  
	ACALL    Read
	MOV     A,DatabusL
	ANL     A,#010001000B
	JNZ     Soft_Reset      ;Wait for BSY=0, DRQ=0
	ACALL    Set_Command
	MOV     DDL,#008h       ;Soft Reset
	MOV     DDH,#000h
	ACALL    Write
	ACALL    DELAY
	RET    
;========================== Get_Buttons
Get_Buttons:
	MOV     A,P3
	ANL     A,#011110000B
	CJNE    A,Button_State,but
	MOV     A,P1
	ANL     A,#010000000B
	CJNE    A,Button_State1,but
	ACALL	b5
but:  ACALL    Sub_Channel		;Read current Audio Status and Track Position
	MOV     A,P3   
	ANL     A,#011110000B
	MOV     Button_State,A
	MOV     A,P1
	ANL     A,#010000000B		;Pause button is on the P1.7
	MOV     Button_State1,A
b0:   MOV     A,P3
	ANL     A,#000010000B
	CJNE    A,#000010000B,b1
	ACALL   Play_Button
	ACALL	b5
b1:   MOV     A,P3
	ANL     A,#000100000B
	CJNE    A,#000100000B,b2
	ACALL   Stop_Button
	ACALL	b5
b2:   MOV     A,P3
	ANL     A,#001000000B
	CJNE    A,#001000000B,b3
	ACALL   Rev_Button
	ACALL	b5
b3:   MOV     A,P3
	ANL     A,#010000000B
	CJNE    A,#010000000B,b4
	ACALL   Fwd_Button
	ACALL	b5
b4:   MOV     A,P1
	ANL     A,#010000000B
	CJNE    A,#010000000B,b5
	ACALL   Pause_Button
b5:   
	MOV	A,#000H
	MOV	Button_State,A
	MOV	Button_State1,A
	MOV	P1,A
	MOV	P3,A
	RET
Play_Button:
			ACALL    Read_TOC
			MOV     M_Field,#000H
			MOV     S_Field,#002H
			MOV     F_Field,#000H
			ACALL    Play
			RET
Stop_Button:	ACALL    Stop_Play
			RET
Fwd_Button: 	MOV     DPH,#00FH   
			MOV     DPL,#000H   ;Last 256 Bytes of memory
			MOV     A,Track
			INC     A
			MOV     Track,A
Get_Fwd_MSF:   	MOV     A,DPL
			INC	  A
			MOV	  DPL,A
			MOVX    A,@DPTR
			MOV     M_Field,A
			MOV     A,DPL
			INC	  A
			MOV	  DPL,A
			MOVX    A,@DPTR
			MOV     S_Field,A
			MOV     A,DPL
			INC	  A
			MOV	  DPL,A
			MOVX    A,@DPTR
			MOV     F_Field,A
			MOV     A,Track
			DEC     A
			MOV     Track,A
			MOV     A,Track
			JNZ     Get_Fwd_MSF     ;Loop until desired track reached           
			ACALL    Play   
			RET
Rev_Button: 	MOV 	  DPH,#00FH   ;
			MOV     DPL,#000H   ;Last 256 Bytes of memory
			MOV     A,Track
			DEC     A
			MOV     Track,A
Get_Rev_MSF:  	MOV     A,DPL
			INC	  A
			MOV	  DPL,A
			MOVX    A,@DPTR
			MOV     M_Field,A
			MOV     A,DPL
			INC	  A
			MOV	  DPL,A
			MOVX    A,@DPTR
			MOV     S_Field,A
			MOV     A,DPL
			INC	  A
			MOV	  DPL,A
			MOVX    A,@DPTR
			MOV     F_Field,A
			MOV     A,Track
			DEC     A
			MOV     Track,A
			MOV     A,Track
			JNZ     Get_Rev_MSF ;Loop until desired track reached       
			ACALL    Play
			RET
Pause_Button:
		ACALL    Pause
		RET
;========================== Send Packet Command (ATA)
Send_Packet:
	ACALL    Set_Status  
	ACALL    Read
	MOV     A,DatabusL
	ANL     A,#010001000B
	JNZ     Send_Packet         ;Wait for BSY=0, DRQ=0
	ACALL    Set_DriveSelect
	MOV     DDL,#011100000B     ;Select Drive 0, LBA
	MOV     DDH,#00H
	ACALL    Write        
	ACALL    Set_ByteCountLow
	MOV     DDL,#0FFH           ;Maximum 255!!!!!       
	ACALL    Write
	ACALL    Set_ByteCountHigh
	MOV     DDL,#0FFH		 ;Maximum 255!!!!!  
	ACALL    Write
	ACALL    Set_Feature
	ACALL    Write
	ACALL    Set_Command
	MOV     DDL,#0A0H               ;ATAPI Packet Command
	ACALL    Write
Wait_DRQ:
	ACALL    Set_Status      
	ACALL    Read
	MOV     A,DatabusL
	ANL     A,#010001000B
	CJNE    A,#000001000B,Wait_DRQ  ;Poll for DRQ=1,BSY=0
	RET
;========================== Start_Unit Packet
Start_Unit:
	ACALL    Send_Packet
	ACALL    Set_Data
	MOV     DDL,#01BH       ;Operation Code
	MOV     DDH,#000H       ;Reserved, Bit0,Immed = 0
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#001H       ;Reserved, Bit1,LoEj = 0, Bit0,Start = 1
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	ACALL    Write
	ACALL    Write
	ACALL    Wait_INTRQ
	RET
;========================== Stop Play Packet
Stop_Play:
	ACALL    Send_Packet
	ACALL    Set_Data
	MOV     DDL,#04EH       ;Operation Code
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	ACALL    Write
	ACALL    Write
	ACALL    Write
	ACALL    Write
	ACALL    Write
	ACALL    Wait_INTRQ
	RET
;========================== Play Packet
Play:   
	ACALL    Send_Packet
	ACALL    Set_Data
	MOV     DDL,#047H       ;Play Audio MSF Command
	MOV     DDH,#000H       ;Resered
	ACALL    Write
	MOV     DDL,#000H       ;Resered
	MOV     A,M_Field
	MOV     DDH,A           ;Starting M Field
	ACALL    Write
	MOV     A,S_Field
	MOV     DDL,A           ;Starting S Field
	MOV     A,F_Field
	MOV     DDH,A           ;Starting F Field
	ACALL    Write
	MOV     DDL,#063H       ;Ending M Field  
	MOV     DDH,#03BH       ;Ending S Field
	ACALL    Write
	MOV     DDL,#04AH       ;Ending F Field
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	ACALL    Write
	ACALL    Wait_INTRQ       
	RET
;========================== Pause/Resume Packet
Pause:  
	ACALL    Send_Packet 
	ACALL    Set_Data
	MOV     DDL,#04BH       ;Pause/Resume Command
	MOV     DDH,#000H       ;Resered
	ACALL    Write
	MOV     DDL,#000H       ;Resered
	ACALL    Write
	ACALL    Write
	ACALL    Write
	;Check if already paused
	MOV     DDL,#000H       ;Reserved, Resume Bit(0) (Pause if playing)
	MOV     A,Audio_Status
	CJNE    A,#012H,Paused
	MOV     DDL,#001H       ;Resume if Paused
Paused:     
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	ACALL    Write
	ACALL    Wait_INTRQ       
	RET
;========================== Read Sub-Channel Packet
Sub_Channel:    
	ACALL    Send_Packet
	ACALL    Set_Data
	MOV     DDL,#042H       ;Read Sub-Channel Command
	MOV     DDH,#002H       ;Resered, Bit1=MSF, Bit0=Reserved
	ACALL    Write
	MOV     DDL,#040H       ;Resered, Bit6=SubQ, Reserved
	MOV     DDH,#001H       ;Sub-cahnnel Data Format (CD-ROM current position)
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Track number (Must be zero - Not relevant)
	MOV     DDH,#000H       ;Allocation Length (MSB) (!!!!!! I think zero!)
	ACALL    Write
	MOV     DDL,#001H       ;Allocation Length (LSB) (!!!!!! I think ONE!)
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	ACALL    Wait_INTRQ   
	ACALL    Read_Channel_Data   
	RET
;========================== Read TOC Packet
Read_TOC:    
	ACALL    Send_Packet
	ACALL    Set_Data
	MOV     DDL,#043H       ;Read Sub-Channel Command
	MOV     DDH,#002H       ;Resered, Bit1=MSF, Bit0=Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Reserved, Format
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Starting Track
	MOV     DDH,#000H       ;Allocation Length (MSB) 
	ACALL    Write
	MOV     DDL,#001H       ;Allocation Length (LSB) 
	MOV     DDH,#000H       ;Format, Reserved
	ACALL    Write
	MOV     DDL,#000H       ;Reserved
	MOV     DDH,#000H       ;Reserved
	ACALL    Write
	ACALL    Wait_INTRQ   
	ACALL    Read_TOC_Data   
	RET
;========================== Wait for INTRQ=1 Pending Command Packet
Wait_INTRQ:        
	MOV     A,P3
	ANL     A,#000000100B
	JZ      Wait_INTRQ
	ACALL    Set_Status
	ACALL    Read
	ACALL    Set_CauseOfInt      ;To clear the interupt for Data In
	ACALL    Read 
	RET
;========================== Read in Sub Q Data
Read_Channel_Data:
	;Wait for DRQ=1
	ACALL    Set_Status
	ACALL    Read
	MOV     A,DatabusL
	ANL     A,#000001000B   ;Wait DRQ=1
	JZ      Read_Channel_Data
	ACALL    Set_Data
	ACALL    Read
	MOV     A,DatabusH       ;Bit 1
	MOV     Audio_Status,A
	ACALL    Read
	ACALL    Read
	ACALL    Read
	MOV     A,DatabusL       ;Bit 6
	MOV     Track,A
	ACALL    Read
	ACALL    Read
	ACALL    Read
	ACALL    Read
	;Should be a DRQ=0 NOW
	ACALL    Set_Status
	ACALL    Read
	RET
;========================== Read in TOC Data
Read_TOC_Data:  
	;Wait for DRQ=1
	ACALL    Set_Status
	ACALL    Read
	MOV     A,DatabusL
	ANL     A,#000001000B   ;Wait DRQ=1
	JZ      Read_TOC_Data
	ACALL    Set_Data
	ACALL    Read
	MOV     A,DatabusH       ;Bit 1
	MOV     TOC_Length,A    ;LIMITATION - Only reads ONE byte of the length!
				;              IE Less that 255 bytes 36 Tracks MAX 
	ACALL    Read
	MOV     A,DatabusL
	MOV     First_Track,A
	MOV     A,DatabusH
	MOV     Last_Track,A
	DEC     TOC_Length
	DEC     TOC_Length
	;ALLOCATE THE FIRST PORTION OF MEMORY HERE
	MOV     DPH,#00FH   ;
	MOV     DPL,#000H   ;Last 256 Bytes of memory
Next_Descriptor:
	ACALL    Read
	ACALL    Read    
	ACALL    Read
	;M Field
	MOV     A,DPL
	INC	  A
	MOV	  DPL,A
	MOV     A,DatabusH
	MOVX    @DPTR,A
	ACALL    Read
	;S Field
	MOV     A,DPL
	INC	  A
	MOV	  DPL,A
	MOV     A,DatabusL
	MOVX    @DPTR,A 
	;F Field
	MOV     A,DPL
	INC	  A
	MOV	  DPL,A
	MOV     A,DatabusH
	MOVX    @DPTR,A 
	DEC     TOC_Length
	DEC     TOC_Length
	DEC     TOC_Length
	DEC     TOC_Length
	DEC     TOC_Length
	DEC     TOC_Length
	DEC     TOC_Length
	DEC     TOC_Length
	MOV     A,TOC_Length
	JNZ     Next_Descriptor
	;Should be a DRQ=0 NOW
TOC_DRQ_Wait:    
	ACALL    Set_Status      
	ACALL    Read
	MOV     A,DatabusL
	ANL     A,#000001000B
	CJNE    A,#000000000B,TOC_DRQ_Wait  ;Poll for DRQ=0
	RET
;========================== Set Addresses
Set_Feature:
Set_CauseOfInt:   CLR     CS0     
			SETB    CS1    
			CLR     DA2     
			SETB    DA1    
			CLR     DA0    
			RET
Set_ByteCountLow: CLR     CS0
			SETB    CS1
			SETB    DA2
			CLR     DA1
			CLR     DA0
			RET
Set_ByteCountHigh:CLR     CS0
			SETB    CS1
			SETB    DA2
			CLR     DA1
			SETB    DA0
			RET
Set_DriveSelect:  CLR     CS0     
			SETB    CS1     
			SETB    DA2     
			SETB    DA1     
			CLR     DA0     
			RET
Set_Command: 
Set_Status:       CLR     CS0   
			SETB    CS1    
			SETB    DA2     
			SETB    DA1    
			SETB    DA0    
			RET
Set_Data:         CLR     CS0
			SETB    CS1
			CLR     DA2
			CLR     DA1
			CLR     DA0
			RET
;========================== Read Databus
Read:       MOV     DDL,#0FFH       ;Port0 Pull-ups
		CLR     DIOR            ;Assert DIOR
		MOV     A,DDL
		MOV     DatabusL,A
		MOV     A,DDH
		MOV     DatabusH,A
		SETB    DIOR            ;Negate DIOR
		MOV     DDH,#000H
		RET
;========================== Write Databus
Write:      NOP
		CLR     DIOW            ;Assert DIOW
		NOP
		SETB    DIOW            ;Negate DIOW
		RET
;========================== Delay
DELAY:      MOV     R4,#00AH
DELAY1:     MOV     R2,#0A0H
D2:         MOV     R3,#000H
D3:         XCH     A,R5
		XCH     A,R5
		DJNZ    R3,D3
		DJNZ    R2,D2
		DJNZ    R4,DELAY1      
		RET         
END
